#!/usr/bin/perl -w
#
# This script needs to be run when the ip address changes.
#
# On RedHat, this is possible by adding it to the end of
#	/etc/sysconfig/network-scripts/ifup-post
# On Debian, this is possible by inserting 'up' lines in
#	/etc/network/interfaces for the interfaces we are monitoring.
#
# Yes, I know. This script is amazingly hackish. That is due to poor planning
# on my part before I began. The structure of the @services array is horrible.
# The Reverse DNS handling is *really* ugly. The rest may be okay.
#
# New "handlers" can be installed in the following fashion:
# 1. Add the tag name to ipchange.pm in the get_defaults and the
#	ipchange_names functions
# 2. Add the service's information to the @services array and the
#	%restart hash, or create your own handler (eg: DNS).

#use strict;
use nameshash;
use File::Copy qw{cp mv}; # For file manipulation
# The actual equivalent of "use ipchange", written out in
# full to specify an absolute path. The ipchange.pm cotains the config file
# routines that are used by both this script and ipchange.cgi, the UI.
BEGIN {
	require "$OS_WEB_PLUGINS/os-dyndns/ipchange.pm";
	'ipchange'->import();
	}

my $ifconfig = '/sbin/ifconfig'; # ifconfig path
my $DEBUG = 0; # Set to 1 to output information on what it is doing
my $DB = "/etc/httpd/logs/error_log";

# When $DEBUG=0, the the program will run only if the lock file
# /var/lock/ipchange exists
if (!($DEBUG or -e "/var/lock/ipchange")) {
	#print "I quit.\n";
	exit 0;
}

open DB, ">> $DB" if $DEBUG;
print DB "$0\n" if $DEBUG;

my %old = ipchange_config (); # Stores the previous configuration
my ($xx, $yy, $key, $value);
if ($DEBUG) {
	foreach $key (values %old) {
		print DB "\t key: $key.\n";
		if ($key =~ /ARRAY/) {
			foreach $xx (@$key) {
				print DB "\t $xx ";
			}
		} else {
			foreach $xx (%$key) {
				print DB "\t $xx ";
			}
		}
		print DB "\n";
	}
}
my %new = ipchange_config (); # Stores the new configuration
require "$OS_CGI_DIR/common.pl";
my %passwdHash = read_passwd();

# check if the email address is still valid, or set to admin name
if (!exists $passwdHash{$new{email}} and $new{email} !~ /.*@.*/) {
	$new{email} = get_admin_name();
}

## if ($DEBUG) {
	## foreach (keys %new) {
		## print DB "new{$_}: $new{$_}.\n";
		## foreach $yy (@$new{$_}) {
			## print DB "\t $yy ";
		## }
		## print DB "\n";
	## }
## }

my %public; # Put all the public interfaces here
open (FH, $NETWORK_INFO_FILE) or die $!;
## map ( {$public{$1} = 1 if ( /^(eth\d+)-access=public/ );} <FH>);
map ( {$public{$1} = 1 if ( /^(eth\d+)-access/ );} <FH>);

## nope # Only run for public interfaces
## if ( $#ARGV == 0 and ! defined $public{$ARGV[0]} )
## {
	## print DB "Ignoring non-public network interface.\n" if ( $DEBUG );
	## exit (1);
## }
## 
## # what happens when there's no public interfaces???

my @interfaces = keys %public;
my @ppp = get_ppp_dev();
print DB "\t 1. interfaces: @interfaces.\n" if $DEBUG;
print DB "\t ARGV[0] has $#ARGV params.\n" if $DEBUG;
if ( $#ARGV == 0 ) {
	print DB "\t params...\n" if $DEBUG;
	$ARGV[0] =~ /(.*)/;
	$ARGV[0] = $1;
	@interfaces = ($ARGV[0]);
} else {	
	## if params aren't specified, add the ppp devices too.
	print DB "\t no params...\n" if $DEBUG;
	foreach (@ppp) {
		push @interfaces, $_;
	}
}
## push @interfaces, get_ppp_dev ();
## print DB "2. interfaces: @interfaces.\n" if $DEBUG;
if ($DEBUG) {
	print DB "\t Interfaces: ";
	foreach (@interfaces) {
		print DB "$_ ";
	}
	print DB ".\n";
}

my @changed; # Stores the names of interfaces which had an IP change
foreach my $interface (@interfaces)
{
	print DB "\t interface: $interface.\n" if $DEBUG;
	if ( open (PIPE, "$ifconfig $interface|") )
	{
		my $line = <PIPE>; # Toss the first line
		$line = <PIPE>; # The interesting stuff is in the second line
		close (PIPE);
		
		$new{$interface}{ip} = $1 if ( $line =~ /inet addr:(\d+\.\d+\.\d+\.\d+)/ );
		$new{$interface}{netmask} = $1 if ( $line =~ /Mask:(\d+\.\d+\.\d+\.\d+)/ );
	}
	else
	{
		die "Could not run ipconfig: $!\n";
	}

	# Decide if we need to do anything
	if (exists ($old{$interface})) {
		print DB "\t old{$interface}{ip}: $old{$interface}{ip}.\n" if $DEBUG;
		print DB "\t new{$interface}{ip}: $new{$interface}{ip}.\n" if $DEBUG;
		print DB "\t (they should be different if IP changed...)\n" if $DEBUG;
		print DB "\t old{$interface}{netmask}: $old{$interface}{netmask}.\n" if $DEBUG;
		print DB "\t new{$interface}{netmask}: $new{$interface}{netmask}.\n" if $DEBUG;
	} else {
		print DB "\t old{$interface} doesn't exist.\n" if $DEBUG;
	}

	if ( exists $old{$interface} and 
	     $old{$interface}{ip} eq $new{$interface}{ip} and
		 $old{$interface}{netmask} eq $new{$interface}{netmask} )
	{
		print DB "\t The IP address has not changed.\n" if ( $DEBUG );
	}
	elsif (!exists $old{$interface} and !exists $new{$interface})
	{
		print DB "\t Interface: $interface is not configured.\n" if ($DEBUG);
	}
	else
	{
		push (@changed, $interface);
	}
}

# Write out the current IP address(es)
ipchange_write_config( %new ) or die "Could not write configuration: $!\n";

my @warnFiles = ();
my @collisionFiles = ();
my @reportFiles = ();

# Maps file names to service names
my %fileToService = (
	'/etc/ftpaccess' => 'Virtual FTP Hosts',
	);
	
my %tagNames = ipchange_names();
	
# restart is a hash of lists of commands which should be executed when the
# service is updated. The keys can be services or file names.
my %restart = (
	'firewall' => ['/etc/rc.d/init.d/firewall restart'],
	'portfw' => ['/etc/rc.d/init.d/portfw start'],
	'ipsec' => ['/etc/rc.d/init.d/ipsec restart'],
	'/etc/rc.d/rc.virtual' =>
		["$OS_WEB_PLUGINS/os-dyndns/virtual restart"],
		#'/etc/rc.d/init.d/virtual start'], # A different one is used to stop it because we need to use the old data
	'fdns' => ['/etc/rc.d/init.d/named restart'],
	'rdns' => ['/etc/rc.d/init.d/named restart'],
	'remote' => ['/etc/rc.d/init.d/vsgate restart'],
	'virtualhost' => ['/etc/rc.d/init.d/httpd reload'],
	'dhcpd' => ['/etc/rc.d/init.d/dhcpd restart'],
	);
		
# stop is a hash of lists of commands which should be executed before the search-replace happens.
my %stop = (
	'portfw' => ['/etc/rc.d/init.d/portfw stop'],
	);

# Do the update for each interface which changed
foreach my $interface (@changed)
{
	# This files will be used for each of the changed interfaces, in the e-mail report. 
	@warnFiles = ();
	@collisionFiles = ();
	@reportFiles = ();

	# Quit if this is the first run for this interface
	if ( ! exists $old{$interface} )
	{
		print DB "\t Current settings saved.\n" if ( $DEBUG );
		next;
	}
	
	# Now that the configuration is written we can take some shortcuts
	$old{ip} = $old{$interface}{ip};
	$old{netmask} = $old{$interface}{netmask};
	$new{ip} = $new{$interface}{ip};
	$new{netmask} = $new{$interface}{netmask};
	
	# Calculate the network addresses
	$old{network} = join( '.', unpack( 'C*',
		pack( 'C*', split( /\./, $old{ip} )) & pack( 'C*', split( /\./, $old{netmask} )) ));
	$new{network} = join( '.', unpack( 'C*',
		pack( 'C*', split( /\./, $new{ip} )) & pack( 'C*', split( /\./, $new{netmask} )) ));
	
	# Search and Replace
	# Everything is essentially a search and replace.
	# services is a 3D array that stores data in the following format:
	#	[ service tag, [filename, [search regexp, replacement expression], ... ], ... ]
	# Essentially, service => files => search,replace pairs, all of which are one to many relationships.
	# - The order of the rules is important: IPSEC must be brought up *after* the firewall reset
	# - Make the rules as specific as possible to minimize the chance of a false match
	# - Backslash must be escaped! \ => \\
	my @services = (
		# Virtual Hosts
		#[ 'virtualhost',
		#	[ '/etc/ftpaccess',
		#		[ "^\\s*virtual\\s+$old{ip}\\b", "virtual $new{ip}"],
		#	],
		#	[ '/etc/rc.d/rc.virtual',
		#		[ "\\b$old{ip}\\b", "$new{ip}"],
		#	],
		#	[ '/etc/httpd/conf/httpd.conf',
		#		[ "\\b$old{ip}\\b", "$new{ip}"],
		#	],
		#],
		
		# Host name aliases
		[ 'hosts',
			[ '/etc/hosts',
				[ "^\\s*$old{ip}\\b", "$new{ip}"],
			],
		],
	
		#	
		[ 'dhcpd',
			[ '/etc/dhcpd.conf',
				[ "\\b$old{ip}\\b", "$new{ip}"],
			],
		],

		# Firewall rules
		[ 'firewall',
			[ '/etc/rc.d/rc.firewall',
				[ "\\b$old{ip}", "$new{ip}"],
				[ "\\b$old{network}/$old{netmask}\\b", "$new{network}/$new{netmask}"],
			],
		],
		
		# Port forwarding rules
		[ 'portfw', 
			[ '/etc/rc.d/rc.portfw',
				[ "\\b$old{ip}\\b", "$new{ip}" ],
				[ "\\b$old{network}/$old{netmask}\\b", "\\b$new{ip}/$new{netmask}\\b" ],
			],
		],
		
		# IPSec: This may need a special handler at some point because the config file
		#	can include other config files. Right now that feature is not used in RebelConnect.
		# These rules *must* be after the firewall rules
		[ 'ipsec',
			[ '/etc/ipsec.conf',
				[ "\\b$old{ip}\\b", "$new{ip}" ],
			],
			[ '/etc/ipsec.secrets',
				[ "\\b$old{ip}\\b", "$new{ip}" ],
			],
		],
	
		);
		
	# Foward DNS
	# Forward DNS is easy: Just search and replace in all the forward dns files
	# The problem is that those files can change
	opendir (DIR, '/var/named') or die "$!";
	my @files = grep (! (/in-addr.arpa/ or /^\.\.?$/ or /~$/) , readdir (DIR) ); # Exclude reverse DNS
	closedir (DIR);
	
	my @searches = ();
	# Search every file for the old ip, replacing it with the new ip
	foreach my $file (@files)
	{
		push (@searches,  ["/var/named/$file", [ "A\\s+$old{ip}\\b", "A\t$new{ip}" ]]);
	}
	# Search the /etc/resolv.conf as well
	push (@searches, ["/etc/resolv.conf", [ "\\b$old{ip}\\b", "$new{ip}" ]]);
	
	push (@services, [ 'fdns', @searches ]); # Add the searches to the service

	# Search replace virtualhost related files. This needs to take place after fdns.
	@searches = ();
	push (@searches, ['/etc/ftpaccess', [ "^\\s*virtual\\s+$old{ip}\\b", "virtual $new{ip}"]]);
	push (@searches, [ '/etc/rc.d/rc.virtual', [ "\\b$old{ip}\\b", "$new{ip}"]]);
	push (@searches, [ '/etc/httpd/conf/httpd.conf', [ "\\b$old{ip}\\b", "$new{ip}"]]);

	push (@services, ['virtualhost', @searches]); # Add the 'virtualhost service after 'fdns'
	
	# Go through all the services, replacing where it is needed
	print DB "\tChecking services...\n" if $DEBUG;
	foreach my $serviceRow (@services)
	{
		my ($service, @files) = @$serviceRow;
		
		print DB "\t\t service: $service.\n" if $DEBUG;
		if (! (exists $new{$service} and
		    (${$new{$service}}[0] or ${$new{$service}}[1]) ) ) {
			print DB "\t\tskipping to the next service\n" if $DEBUG;
		}
		# Skip this service if the tag doesn't exist in the config file
		# or if neither "Update" or "Report" are selected.
		next if ( ! (exists $new{$service} and
			(${$new{$service}}[0] or ${$new{$service}}[1]) ) );
			
		my $masterCount = 0;
		my $totalCount = 0;
		
		# If the file contains both the new IP and the old IP report the "collision"
		# and enter the files in the fileService hash by mapping the file -> tag -> name
		foreach my $fileRow (@files)
		{
			my $file = $$fileRow[0];
			
			if ( ! exists $fileToService{$file} and exists $tagNames{$service} )
			{
				$fileToService{$file} = $tagNames{$service};
			}
			
			# count the number of occurences of the new ip
			# in each file before changing the file. If the
			# new ip is already in there, check to see if the
			# old ip is there too. If both ips are in the file,
			# there was a 'collision', save the filename for
			# later reporting.

			# if you are changing from ...48.177 and
			# ...48.17, 17 will match 177, so have to append
			# the word boundary (\\b) to the string.

			my $count = search( $file, ["\\b" . $new{ip} . "\\b"] );
			my $count0 = search( $file, ["\\b" . $old{ip} . "\\b"] );

			warn ("Search $file failed: $!\n"), next if ( ! defined $count );

			$totalCount += $count0;
			if ( $count > 0 and $count0 > 0 )
			{
				$masterCount = 1;
				push( @collisionFiles, $file );
			}
		}
		# Don't update this service if there is a collision
		next if ( $masterCount > 0 );
		
		# No problem, do we need to stop the service before making any search/replace?
		restart( $stop{$service} ) if
                        ( $new{$service}[0] and $totalCount > 0 and exists $stop{$service} );

		$masterCount = 0;
		foreach my $fileRow (@files)
		{
			my ($file, @regexpPairs) = @$fileRow;
			
			my $count = 0;
			# Update
			if ( $new{$service}[0] )
			{	
				# Do the replacement
				$count = searchreplace ($file, @regexpPairs);
				
				warn ("Search and replace $file failed: $!\n"), next if ( ! defined $count );
				
				$masterCount += $count;
				print DB "\t $0: reportFiles: GOT HERE!\n" if ($DEBUG);
				print DB "\t file: $file.\n" if ($DEBUG);
				print DB "\t count: $count (should be > 0).\n" if ($DEBUG);
				push( @reportFiles, $file ) if ( $count > 0 and $new{$service}[1] );
				
				# modify the serial# of the file if the service is fdns and $count > 0
				modify_serial($file) if ($service eq "fdns" and $count > 0);

				# run the restart command if we are supposed to
				restart( $restart{$file} ) if ( $count > 0 and exists $restart{$file} );
			}
			# Warn
			else
			{
				# Do the search
				$count = search ($file, @regexpPairs);
				
				warn ("Search $file failed: $!\n"), next if ( ! defined $count );
				
				# Add the file to the email warning
				push (@warnFiles, $file) if ( $count > 0 );
			}
			
			print DB "\t $file had $count matches\n" if ( $DEBUG );
		} # End of foreach $file
		
		# run the restart command if we are supposed to
		sleep 2 if ($service eq 'portfw'); # make sure the replace file saved to disk
		restart( $restart{$service} ) if
			($new{$service}[0] and $masterCount > 0 and exists $restart{$service}
			 and $service ne 'fdns');
		# only restart fdns in a specific situation
		restart( $restart{$service} ) if
			($service eq 'fdns' and $new{$service}[0] and $masterCount > 0 and exists $restart{$service}
			 and $new{virtualhost}[0]);
	}
	
	# Reverse DNS
	if ( $new{rdns}[0] or $new{rdns}[1] )
	{
		require "$OS_CGI_DIR/bind.pl";
		require "$OS_CGI_DIR/common.pl";
		
		rdns();
		#restart( $restart{rdns} ) if 
		#	($interface =~ /ppp[0-2]/ and ppp_listen() == 0 and $new{rdns}[0]
		#	 and exists $restart{rdns});
	}

	# Rebel Remote
	if ($new{remote}[0])
	{
		print DB "\t Calling /var/www/vtcpbin/scriptgen.sh.\n" if $DEBUG;
		system("/var/www/vtcpbin/scriptgen.sh") if(-x "/var/www/vtcpbin/scriptgen.sh");
		restart( $restart{remote} );
	}

	# Email Report
	if (@warnFiles or @collisionFiles or @reportFiles ) {
		email_report($interface);
	}
} # End of foreach changed interface

# Email Reporting
sub email_report {
	my $iface = $_[0];
	# debug messages
	if ( @warnFiles or @collisionFiles or @reportFiles ) {
		print DB "\t ipchange will send an email.\n" if $DEBUG;
	} else {
		print DB "\t ipchange will NOT send an email.\n" if $DEBUG;
	}

	if ( @warnFiles or @collisionFiles or @reportFiles )
	{
		# The messages array stores an array reference to the file list
		# and a message to put before the file list
		my @messages = ( [ \@collisionFiles, <<EOF ],
-- IP Address Conflicts --

The following files contain both the old and new IP addresses so they were
not automatically updated with the new settings. The services they belong
to may need reconfiguration.

EOF
		[ \@warnFiles, <<EOF ],
-- Warnings --

The following files contain the old IP address. The services they belong to
may need reconfiguration.

EOF
		[ \@reportFiles, <<EOF ],
-- Updated Files --

The following files contained the old IP addresses and were updated with
the new settings.

EOF
		);
	
		# Let's run sendmail and send it to them
		print DB "\t Sending email to $new{email}\n" if ( $DEBUG );
		# The options to sendmail are
		# -i		ignore lines with only a "." on them
		# -t		Find the recipients from the message contents
		# -odb	Option: DeliveryMode: Background - deliver in background
		# -U	Initial user submission: According to the man page, should be set when
		#		called from a user agent. This is essentially what we are doing
		print DB "\t Sending mail to $new{email}.\n" if $DEBUG;
		open( SENDMAIL, "|/usr/sbin/sendmail -i -t -odb -U" ) or die "$!";
		print SENDMAIL <<EOF;
From: IP Change Notification <root>
To: <$new{email}>
Subject: OfficeServer IP Address Change Report

This message has been automatically generated by the IP Change Reporter 
to notify you that the OfficeServer's IP address has changed from
$old{ip} to $new{ip} on your $iface interface and your current network
configuration may be incorrect.

EOF
		foreach my $section ( @messages )
		{
			next if ( ! @{$$section[0]} ); # Only print the sections we need
			print SENDMAIL $$section[1], reportFiles( $$section[0], \%fileToService ), "\n";
		}	

		close( SENDMAIL ) or warn "$!";
	}
}
close DB;

# Searches a given file for a regular expression replacing it with
# the given string. The regular expression will be searched with the
# global (s///g) option. A backup will be kept with the same filename
# with a ~ appended
#
# Arguments:
#	string - file to search and replace
#	list of lists - [ [ regexp, replacement], ... ]
#
# Return value:
#	integer - Number of replacements
#	undef - Failure
sub searchreplace ($$$) {
	my ($file, @searches) = @_;
	my @file = (); # Will store the entire file
	
#	my $mode = (stat( $file ))[2];
#	print "$mode\n";

	open (IN, $file) or return undef;
	# Make a backup
	cp ($file, "$file~") or return undef;
	open (IN, "$file~") or unlink( "$file~" ), return undef;
	open (OUT, ">$file") or unlink ("$file~"), close (IN), return undef;
	
	# Precompile the regular expressions
	foreach my $search (@searches)
	{
		$$search[0] = qr{$$search[0]};
	}
	
	my $count = 0;
	# Search and replace
	while ( my $line = <IN> )
	{
		foreach my $search (@searches)
		{
			$count += $line =~ s/$$search[0]/$$search[1]/g;
		}
		print OUT $line;
	}
	close (IN);
	close (OUT);
	
	unlink("$file~");
	return $count;
}

# Searches a given file for a regular expression
#
# Arguments:
#	string - file to search
# 	list of lists - a 2D array of regular expressions (see searchreplace)
#
# Return value:
#	integer - Number of times the regular expression matched
#	undef - Failure
sub search ($@) {
	my ($file, @searches) = @_;
	
	return undef if (! -e $file);
	open (IN, $file) or return undef;
	
	# Precompile the regular expression and discard the replacement
	foreach my $search (@searches)
	{
		$search = qr{$$search[0]};
	}
	
	my $count = 0;
	# Search
	while ( my $line = <IN> )
	{
		foreach my $search (@searches)
		{
			while ( $line =~ /$search/g ) # Count ALL matches
			{
				++ $count;
			}
		}
	}
	
	close (IN);
	
	return $count;
}

# Prints out a list of files, organized by category
#
# Arguments
#	array reference - list of files
#	hash reference - Maps files to sections
#
# Return value:
#	string - The formatted report
sub reportFiles (\@\%)
{
	my %sections = ();
	my ($fileNames, $sectionHash) = @_;
	
	# Look for duplicate file names
	@$fileNames = sort @$fileNames;
	for (my $count = 0; $count < $#$fileNames; ++ $count)
	{
		while ( $count < $#$fileNames and $$fileNames[$count] eq $$fileNames[$count + 1] )
		{
			splice( @$fileNames, $count, 1 ); # Cut out one of the elements
		}
	}

	# Build a hash of lists containing section names => [ file names ]
	foreach my $file ( @{$fileNames} )
	{
		# We assume this always exists, which it does in this application
		return undef if ( ! exists $$sectionHash{$file} );
		my $section = $$sectionHash{$file};
		
		# Either create the list, or add it to the list
		if ( ! exists $sections{$section} )
		{
			$sections{$section} = [ $file ];
		}
		else
		{
			push (@{$sections{$section}}, $file);
		}
	}
	
	# Store the report
	my $string = '';
	foreach my $section ( sort keys %sections )
	{
		# The title
		$string .= "$section\n";
		
		foreach my $file ( @{$sections{$section}} )
		{
			$string .= "\t$file\n";
		}
	}
	
	return $string;
}

# Executes a list of commands
#
sub restart ($) 
{
	my ($list) = @_;

	foreach my $command (@$list)
	{
		print DB "\t Running: $command\n" if ( $DEBUG );
		if ( (system ($command) & 0xFFFF) == 0xFF00)
		{
			print STDERR "$command failed: $!\n";
			return;
		}
	}
	
	return;
}

# Reverse DNS Handling
# Reverse DNS is more complicated, so we only do it if we must
#
# The way we fix reverse DNS:
# - Find the zone file for the old IP address
# - Find if that zone file contains the old IP address
# - Find the zone file for the new IP address
# - Find if that zone file contains the new IP address
#	+ Add the files to the conflicts list if both IPs exist
#	+ Warn the user if that's all that is required
#	+ If we need to update, we remove the record and add it again
sub rdns ()
{
	# Find the old ip's reverse DNS file
	my ($oldFile, $oldZone, $oldData) = find_rdns ($old{ip});
	return if ( ! defined $oldData ); # Hey, there is no reverse DNS: No problem

	# Now we have to see if there is a rdns entry for the NEW ip
	my ($newFile, $newZone, $newData) = find_rdns ($new{ip});
	if ( defined $newData )
	{
		# There is both a new and old reverse DNS for the IP: Conflict
		$fileToService{$oldFile} = $tagNames{rdns};
		$fileToService{$newFile} = $tagNames{rdns};
		push (@collisionFiles, $oldFile, $newFile);
		return;
	}
	
	# If all we need to do is warn
	if ( ! $new{rdns}[0] )
	{
		$fileToService{$oldFile} = $tagNames{rdns};
		push (@warnFiles, $oldFile);
		return;
	}
	
	# Keep a backup
	#cp ($oldFile, "$oldFile~");
	#cp ($newFile, "$newFile~");
	
	#Remove the old Reverse DNS entry
	del_zone_record ($oldFile, pseudoNetmask ($old{ip}, $oldZone), $oldData);
	
	# Add a new dns entry
	($newZone, $newFile, $newData) = find_rev_zone($new{ip});
	add_new_zone ($newFile, $newZone) if ( $newData );
	$newFile = "/var/named/$newFile";
	add_zone_record ($newFile, pseudoNetmask ($new{ip}, $newZone), 'PTR', $oldData);
	chmod 0664, "$newFile";
	chown 0,80, "$newFile";
		
	# So we actually need to update it. Too bad, because this takes a bit of work.
	# First things first: Add the files to the warnings lists
	if ( $new{rdns}[1] )
	{
		$fileToService{$newFile} = $tagNames{rdns};
		$fileToService{$oldFile} = $tagNames{rdns};
		push (@reportFiles, $oldFile, $newFile);
	}
	
	# Does the old zone file even have any records in it any more?
	my $soa = 0;
	my $records = 0;
	open (FH, $oldFile) or die "$!";
	while ( defined (my $line = <FH>) and ! $records )
	{
		++ $soa if ( $soa == 0 and $line =~ /SOA.*\(/ );
		++ $soa if ( $soa == 1 and $line =~ /\)/ );
		++ $records if ( $soa == 2 and $line =~ /^.*IN\s*PTR.*$/ );
	}
	close (FH);
		
	if ( ! $records )
	{
		# No records left: Delete the entire zone file, ripped from dnsdel.cgi
		unlink ($oldFile);
		
		open IN, '/etc/named.conf';
		open OUT, ">/etc/named.conf-";
		chown 0,0,"/etc/named.conf-";
		chmod 0644,"/etc/named.conf-";
		my $in_zone = 0;
		my $z;
		while(<IN>) {
			print OUT unless /^zone/;
			if(/^zone\s*\"(\S+)\"/) {
				$z = $1;
				$in_zone = 1;
				print OUT unless $z eq $oldZone;
				while($in_zone) {
					$_ = <IN>;
					if(/^};/) {
						$in_zone = 0;
					}
					next if($z eq $oldZone);
					print OUT;
				}
			}
		}
		close IN;
		close OUT;
		rename "/etc/named.conf-", '/etc/named.conf';
	}
	
	# Let's make sure named.conf is still OK
	chown 0,0,"/etc/named.conf";
	chmod 0644,"/etc/named.conf";

	# And the final touch, restarting the name server
	#restart( $restart{rdns} ); 
}

# Returns undef if there is no RDNS for the ip
# Returns the filename of the zone file it is stored in and the data
# If it exists
sub find_rdns ($)
{
	my ($ip) = @_;
	
	my ($zone, $file, $notThere) = find_rev_zone ( $ip );
	return undef if ( $notThere ); # no reverse DNS
	
	$file = "/var/named/$file";
	
	# Okay, find if the ip is in there
	$ip = pseudoNetmask ($ip, $zone);
	
	# Search the file for it
	$notThere = 1;
	$ip = qr(^$ip); # Precompile the regexp for SPEED
	my $data;
	
	open (FH, $file) or die "$!";
	while ( defined (my $line = <FH>) and $notThere )
	{
		$notThere = 0, $data = $1 if ( $line =~ /$ip\s+IN\s+PTR\s+(\S+)/ ); # Rats, we found it
	}
	close (FH);

	return $file, $zone, undef if ( $notThere ); # No Reverse DNS
	return $file, $zone, $data;
}

# Chops the "zone" part off of an ip, returning the stub as it would appear as
# a reverse DNS entry in that zone's file
sub pseudoNetmask ($$)
{
	my ($ip, $zone) = @_;
	$ip = join '.', reverse split (/\./, $ip); # Reverse the IP
	
	$zone =~ s/\.in-addr\.arpa//; # Chop the "zone" part off of the ip
	$ip =~ s/\.$zone//;
	
	return $ip;
}

sub get_ppp_dev {

my $cmd = "/sbin/ifconfig";
my @data;
my ($i, @ppp) = (0, ());

open PH, "$cmd |";
@data = <PH>;
close PH;

for ($i = 0; $i < $#data; $i++) {
	if ($data[$i] =~ /^(ppp\d+)\s+/) {
		print DB "\t pushing $1 onto the array.\n" if $DEBUG;
		push @ppp, $1;
	}
}
return @ppp;
} # get_ppp_dev

sub ppp_listen {

	my $file = "/etc/ppp/ppp_listen.conf";
	if (-e "$file") {
		open IN, "$file";
		my @lines = <IN>;
		close IN;

		foreach my $line (@lines) {
			return 1 if ($line =~ /^listening=yes/);
		}
	}
	return 0;
}
 
# Modify the Serial# in a zone file if the fdns service has an ip
# change in its entry.
sub modify_serial {
	my $file = $_[0];

	open (IN, $file) or return undef;
	my @lines = <IN>;
	close (IN);

	open (OUT, ">$file");
        my $date = time();
        foreach my $line ( @lines )
        {
                $line =~ s/\d+.*Serial/$date\t; Serial/g;
                print OUT $line;
        }
        close (OUT);

	return 1;
}

