package ipchange;
use Exporter;
use nameshash;

@ISA = ('Exporter');
@EXPORT = qw( &ipchange_config &ipchange_write_config &ipchange_names &ipchange_value_names 
	&ipchange_boolean_options );
@EXPORT_OK = qw( &get_defaults );

my $Config = '/etc/ipchange.conf';

=item get_defaults

	Arguments: None
	Return: The default configuration hash.
	Notes: Not exported by default. Some options are lists, others are scalars (strings).
	Example:
		my %config = OfficeServer::ipchange::get_defaults ();
	
=cut
sub get_defaults ()
{
	my %config = (
		'email' => get_admin_name(),
		'virtualhost' => [ 0, 0 ],
		'fdns' => [ 0, 0 ],
		'rdns' => [ 0, 0 ],
		'hosts' => [ 1, 0 ],
# 		'ipsec' => [ 1, 0 ],
#		'firewall' => [ 0, 0 ],
		'portfw' => [ 0, 0 ],
		'remote' => [ 0, 0 ],
		'dhcpd' => [ 0, 0 ],
		);

	return %config;
}

=item get_admin_name 

A little bit of the old cut and paste from common.pl, put here to avoid weirdness

Looks in the .admin file for the current web admin username. Else returns "$WEB_USER" if not found.

=cut
sub get_admin_name () {
	my $file = $OS_ADMIN_FILE;
	my $return = '';

	return $WEB_USER if(! -e $file);

	open(IN,$file) || die "Cannot read $file\n";
	$return = <IN>;
	close(IN);
	$return =~ /(.*)/;
	$return = $1;
	return $return;
}

=item ipchange_names

	Arguments: None
	Return: A hash mapping configuration options to names.
	Notes:
	Example:
		use OfficeServer::ipchange
		my %configNames = ipchange_names ();
	
=cut
sub ipchange_names ()
{
	my %names = (
		'ip' => _('IP Address'),
		'netmask' => _('Netmask'),
		'eth' => _('Network Interface'),
		'ppp' => _('Point to Point Interface'),
		'email' => _('Address to send report to'),
		'virtualhost' => _('Virtual hosts'),
		'fdns' => _('DNS - Forward domain names'),
		'rdns' => _('DNS - Reverse domain names'),
		'hosts' => _('Host name aliases'),
		'ipsec' => _('RebelConnect'),
		'firewall' => _('Firewall'),
		'portfw' => _('Port forwarding'),
		'remote' => _('RebelRemote'),
		'dhcpd' => _('DHCP update'),
		'lo' => _('Loopback Interface'), # even it doesn't make snese, it dose fix a bug.
		);
		
	if ( open (FH, $NETWORK_INFO_FILE) )
	{		
		map ( {$names{$1} = $2 if ( /^(eth\d+)-desc=(.*)/ );} <FH>);
		close (FH);
	}
	
	return %names;
}

=item ipchange_value_names

	Arguments: None
	Return: A hash mapping numeric option values to names.
	Notes:
	Example:
		use OfficeServer::ipchange
		my %valueNames = ipchange_value_names ();
	
=cut
sub ipchange_value_names ()
{
	my @values = (_('Automatic Update'), _('Report'));
		
	return @values;
}

=item ipchange_value_names

	Arguments: None
	Return: A hash that lists all the options which are boolean options
	Notes:
	Example:
		use OfficeServer::ipchange
		my %booleanOptions = ipchange_boolean_options ();
	
=cut
sub ipchange_boolean_options ()
{
	my %bools = get_defaults();
	while ( my ($key, $value) = each %bools )
	{
		if ( ref( $value ) )
		{
			$bools{$key} = 1;
		}
		else
		{
			delete $bools{$key};
		}
	}
		
	return %bools;
}

=item ipchange_config

	Arguments: None
	Return: A  hash with the current configuration stored in the file/from defaults.
	Notes: The configuration is stored in name = value format
	Example:
		use OfficeServer::ipchange;
		my %config = ipchange_config ();
	
=cut
sub ipchange_config ()
{
	my %config = get_defaults ();
	
	if ( open (CONFIG, $Config) )
	{
		while (<CONFIG>)
		{
			chomp;                  # no newline
			s/#.*//;                # no comments
			s/^\s+//;               # no leading white
			s/\s+$//;               # no trailing white
			next unless length;     # anything left?
			
			my ($name, $value) = split( /\s*=\s*/, $_, 2 );

			# Handling depends on type
			if ( ref $config{$name} eq 'HASH' or $value =~ /ip.*netmask/ )
			{
				%{$config{$name}} = split( /\s+/, $value );
			}
			elsif ( ref $config{$name} eq 'ARRAY' )
			{
				@{$config{$name}} = split( /\s+/, $value );
			}
			else
			{
				$config{$name} = $value;
			}
		}
		close (CONFIG)
	}
	
	return %config;
}
1; # Required to be the last line

=item ipchange_write_config

	Arguments: A hash with the configuration to be stored in the file
	Return: 0 on failure, 1 on success
	Notes: The configuration is stored in name = value format
	Example:
		use OfficeServer::ipchange;
		my %config = ipchange_config ();
		$config{dns} = 0;
		ipchange_write_config (%config);
	
=cut
sub ipchange_write_config (\%) 
{
	my %config = ipchange_config();
	my ($newConfigRef) = @_;
	
	# Store the new values over the default configuration
	while ( my ($name, $value) = each %$newConfigRef )
	{
		$config{$name} = $value;
	}
	
	open (CONFIG, ">$Config") or return 0;
	while ( my ($name, $value) = each %config )
	{
		# Special variable type handling
		if ( ref $value eq 'ARRAY' )
		{
			$value = join( ' ', @$value );
		}
		# Hashes are simply even sized lists, so this will work
		elsif ( ref $value eq 'HASH' )
		{
			$value = join( ' ', %$value );
		}
		
		print (CONFIG "$name = $value\n");
	}
	close (CONFIG);
	
	return 1;
}
