#!/usr/bin/perl -w

# (c) Rebel.com Inc.
# Maintain dynamic DNS accounts

use strict;
use vars qw($LC_DOMAIN $OS_LANG $OS_LANG_DEF $core_lang_dir &__);
use CGI;
use nameshash;

require "$OS_CGI_DIR/common.pl";
require "$OS_CGI_DIR/layout.pl";

# The actual equivalent of "use ipchange", written out in
# full to avoid the taint weirdness
BEGIN {
	require './ipchange.pm';
	'ipchange'->import();
	}

my $DEBUG = 0;

print STDERR "$0:\n" if $DEBUG;

# Untaint the global path variable
$ENV{'PATH'} = '/bin:/usr/bin';
delete @ENV{'IFS', 'CDPATH', 'ENV', 'BASH_ENV'};

my $PLUGIN_NAME = 'os-dyndns';


my $c = common_plugin->new($PLUGIN_NAME);
$c->define_l10n_vars();

# l10n setup:
$c->l10n_setup($PLUGIN_NAME);

# create plug-in help link:
my $help_link = $c->plugin_help_link('os_dyndns',
									 'ddns_overview.html',
									 __('DynamicDNS Help'));

my $jscript = <<EOT;
<!-- // hide from older browsers
function browse (url) {
    var url;
    window.open (url, "", "toolbar=1,statusbar=1,scrollbars=1,resizable=1,width=680,height=500");
}
// End hiding -->
EOT

#This script calls itself
my $myself = CGI::url();

# DNS2Go Configuration.
# - the dns2go config file is "/etc/dyndns/dns2go.conf".
# - the dns2go status file is "/var/dns2go/status.dat". this program
#   uses the pid value stored in this file to decide if the dns2go
#   is running.
# - the lock file: "var/lock/subsys/dns2go". this program creates a
#   lock once a dns2go process is brought up, and removes it when the
#   process is taken down.
#    
# Configuration and other files
my $CONFFILE = "/etc/dyndns/dns2go.conf";
my $STATFILE = "/var/dns2go/status.dat";
my $RUNFILE = "/var/dns2go/runflag";
my $TIMEFILE = "/var/dns2go/uptime";
my $LOGFILE = "/var/log/dns2go.log";

# name, description, status, rc3 prefix, initscript
my @d2gservice = (
 'dns2go', 'DNS2Go', get_dns2go_pid(), '', '/usr/bin/dns2go');
my @dipservice = (
 'ipchange', _('Dynamic IP Change'), '', '', '');

my %modes =
	(
	'main' => [\&do_main, __('DynamicDNS')],
	'cfgddns' => [\&dns2go_config, __('DynamicDNS')],
        'modddns' => [\&dns2go_modify, __('DynamicDNS')],
	'saveddns' => [\&dns2go_save, __('DynamicDNS')],
        'actd2g' => [\&dns2go_action, __('DynamicDNS')],
        'cfgdip' => [\&actions_config, __('DynamicDNS')],
        'edit' => [\&edit, __('DynamicDNS')],
        'save' => [\&save, __('DynamicDNS')],
        'actdip' => [\&ipchange_action, __('DynamicDNS')],
	);

my %desc =
	(
	'hosts' => __('Current Dynamic Hosts')
	);
my $mode = CGI::param('mode') || '';

print STDERR "\t dyndns: query_string: " . CGI::query_string() . ".\n"
if $DEBUG;

$mode = 'main' unless exists $modes{$mode};

# Global variables
my @errors;

my @guiheader = ("${modes{$mode}[1]}", 'admin', 'optionsadmin', 7);

print $c->start_gui_l10n_wrapper(@guiheader) . CGI::script($jscript);
my $result;

# Check username... only httpd should be able to run this
if ($ENV{'REMOTE_USER'} eq get_admin_name() ) {
	$result = &{$modes{$mode}[0]};
} else {
	$result = CGI::center(CGI::b(__('You must be logged in as the administrator to use this plugin.')));
	$result .= $c->return_link_l10n_wrapper('/cgi-bin/netwinder/admin/options.cgi');
}

print $result;
print end_gui();
exit;

## End of normal execution ##

sub do_main {

	my $html ="";
	$html .= h3( $modes{'main'}[1] );

	# Start a form
        $html .= CGI::start_form(-action => CGI::url( -relative=>1 ),
                                 -method => 'get');
	$html .= start_table();
        $html .= CGI::Tr(th([ _('Service'), _('Status'), _('Action')]));

        # Configure DDSN
	$html .= CGI::Tr(te('DNS2Go') . td(status_link(@d2gservice))
                . td(CGI::a({-href=>"$myself?mode=cfgddns"}, _('configure') )));

        # IPChange service
        $html .= CGI::Tr(te(_('Dynamic IP Change')) . td(status_link(@dipservice))
                . td(CGI::a({-href=>"$myself?mode=cfgdip"}, _('configure') )));

        $html .= end_table();
        $html .= CGI::end_form();
        $html .= CGI::p();
        $html .= CGI::center();
        $html .= $help_link;
	$html .= $c->return_link_l10n_wrapper("/cgi-bin/$OS_NAME/admin/options.cgi");
        return $html;
}

# actions_config: this subroutine configures the actions to be performed
# when the host's IP is changed. 

sub actions_config {

        my %config = ipchange_config ();
        my %configNames = ipchange_names ();
        my %passwdHash = read_passwd();

	my $defaultButton;
        my $defaultEmail;

        if (exists $passwdHash{ $config{email} }) {
		$defaultButton = 'local';
	} elsif ($config{email} =~ /.*@.*/) {
		$defaultButton = 'external';
        } else { # local email userid no longer exists, set it to current admin name
		$defaultButton = 'local';
		$config{email} = get_admin_name();
		ipchange_write_config( %config );
	}
	$defaultButton = CGI::param('emailbutton') || $defaultButton;
	
	if ($defaultButton eq 'external' && exists $passwdHash{ $config{email} }) {
       		$defaultEmail = CGI::param('email');
	} else {
		$defaultEmail = CGI::param('email') || $config{email};
	}

        delete $config{email}; # Don't put the email in the next table

        # Start a form
        my $html = h3(_('Dynamic IP Change Configuration'));
        $html .= CGI::start_form(-action => CGI::url( -relative=>1 ),
                                 -method => 'get');
        $html .= CGI::hidden(-override => 1, -name => 'mode',
                             -value => 'edit', -align => 'left');

        my @valueNames = ipchange_value_names();
        my $tbl = start_table();

        #$tbl .= Tr (th ({colspan=>'4'}, _('Actions to Perform')) );
        $tbl .= Tr (th ({-width => '50%'}, _('Service')),
                    th ({-width => '25%'}, \@valueNames ) );

        # Sort by the name order
	delete $config{ppp0}; # a dirty quick fix for bug #3113
        foreach my $option (sort { $configNames{$a} cmp $configNames{$b} } keys %config)
        {
            if (($option !~ /eth[0-2]/) && ($option !~ /ppp[0-2]/) && ($option !~ /lo/)) {
                my @buttons = ();
                for (my $count = 0; $count <= $#valueNames; ++ $count)
                {
                        # I can't use CGI::checkbox_group here since I want them separated into different rows
                        my $endstr = ( ${$config{$option}}[$count] ) ? ' checked>' : '>';
                        push (@buttons, "<input type='checkbox' name='$option' value='$valueNames[$count]'" . "$endstr");
                }

                $tbl .= Tr( te ($configNames{$option}) . td (\@buttons) );
            }
        }

        my %labels = ('local' => _(' use local address'),
                      'external' => _(' use external address') );

        $tbl .= Tr(te({-width=>'50%', -valign=>'top'}, $configNames{email}),
                   te({-width=>'50%', -colspan=>2},
                        CGI::radio_group(-name=>'emailbutton', -override=>1,
                                        -values=> ['local'],
					-default=>$defaultButton,
                                        -labels=>\%labels) .
                        CGI::br() . '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp' .
                        CGI::popup_menu( -name => 'email',
                                        -values => [ sort keys %passwdHash ],
                                        -default => $defaultEmail) .
                        CGI::br() .
                        CGI::radio_group(-name=>'emailbutton', -override=>1,
                                        -values=> ['external'],
                                        -default=>$defaultButton,
                                        -labels=>\%labels) .
                        CGI::br() . '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp' .
                        CGI::textfield(-name=>'extemail',
					-value=>(($defaultButton eq 'external') ? $defaultEmail : ""),
                                        -size=>25,
                                        -maxlength=>60)));
        $tbl .= end_table();

        $html .= $tbl;
        $html .= $c->save_cancel_buttons_l10n_wrapper(CGI::url(-relative=>1));
        $html .= CGI::end_form();

        $html .= $help_link;

        return $html;
}

# This is cut and paste from ipchange.cgi
sub edit () 
{
	my $msg = _('No changes have been made.');
	my $emailbutt = CGI::param('emailbutton');

        $<=0;
        $>=0;
	@errors = ();

	# Configure the other IP change services.
	# Get all the arguments.
	my %args;
	foreach my $arg ( CGI::param() )
	{
		my @list = CGI::param ($arg);
		if ( $#list > 0 )
		{
			$args {$arg} = \@list;
		}
		else
		{
			$args {$arg} = $list[0];
		}
	}
	
	my %edited = argsToConfig( %args );
	$edited{period} = $args{period} if ( exists $args{period} );
	
	#  validate the email address
	if ( $args{'emailbutton'} eq 'external' )
	{
		if ( $edited{email} eq '' )
		{
			push @errors, _('An e-mail address must be specified.');
		}
		elsif ( $edited{email} !~ /^[\w\-\.]+\@[\w\-\.]+[^\.]$/ )
		{
			push @errors, _('Bad e-mail address format.');
		}
	}

        my $html = "";
        if (@errors) {
                $html .= CGI::h3(_('Your request could not be completed because'));
                $html .= CGI::ul(CGI::li(\@errors));
                $html .= CGI::p(CGI::i(_('Please correct the error(s) and try again')));
                $html .= hr();
                return $html . actions_config();
        }

	# Delete unchanged values from the hash
	my %config = ipchange_config();
	
	while ( my ($key, $value) = each %edited )
	{
		my $match = 1;
		# Do we have a list argument?
		if ( ref $value eq 'ARRAY' )
		{
			for (my $count = 0; $count <= $#$value; ++ $count)
			{
				# Hey, they don't match
				$match = 0, last if ( ${$config{$key}}[$count] ne $$value[$count] );
			}
		}
		# If it isn't a reference, and there is a new value
		elsif ( ! ref $value and $config{$key} ne $value )
		{
			$match = 0;
		}
		
		# Delete the key if they match
		delete $edited{$key} if ( $match );
	}

	# If there are new settings
	if ( %edited && !@errors)
	{
        	# Put the arguments into the config hash (i.e. do save)
        	while ( my ($key, $value) = each %edited )
        	{
                	$config{$key} = $value if ( exists $config{$key} );
        	}
        
        	ipchange_write_config( %config ); 

		#my $string = "<p><i>$msg</i></p>";
		#my $string = h3(_('Changed Settings'));
		#$string .= CGI::start_form (-action => CGI::url( -relative=>1 ), -method => 'get') .	
		#		CGI::hidden( -name => 'mode', -value => 'save', -override => 1 );
		#$string .= CGI::hidden(-name=>'emailbutton', -value=>"$emailbutt");
		#$string .= '<p>' . start_table () . #Tr( th( {-width => '100%', -colspan => 5}, _('Changed Settings')) ) .
		#		Tr( th( {-width => '50%'}, _('Parameter') ),
		#			th( {-width => '25%', -colspan => 2}, [_('Old Setting'), _('New Setting')] ) );
		#
		#my %configNames = ipchange_names();
		#
		# Do email first
		#if ( exists $edited{email} )
		#{
		#	$string .= Tr( te({-width => '50%'}, $configNames{email} ) .
		#			te( {-colspan => 2}, [ $config{email}, $edited{email} ] ) ) .
		#			CGI::hidden( -override => 1,
		#						 -name => 'email', -value => ($edited{email})[0] );
		#	delete $edited{email};
		#}
		#
		# Do period next
		#if ( exists $edited{period} )
		#{
		#	$string .= Tr( te({-width => '50%'}, _('Update interval') ) .
		#			te( {-colspan => 2}, [ $config{period}, $edited{period} ] ) ) .
		#			CGI::hidden( -name => 'period', -value => $edited{period} );
		#	delete $edited{period};
		#}
		#
		# Now print a second "header"
		# This one has the labels for "update" and "report"
		#if ( %edited )
		#{
		#	$string .= Tr( th({-width => '50%'}, '&nbsp'), th( {-width => '12%'}, [ ipchange_value_names() ] ) x 2 );
		#	
		#	foreach my $key ( sort keys %edited )
		#	{
		#		$string .= '<tr>' . te( {-width => '50%'}, $configNames{$key} );
		#		foreach my $list ($config{$key}, $edited{$key})
		#		{
		#			foreach my $bool (@$list)
		#			{
		#				$string .= te( ( $bool == 1 ) ? _('Yes') : _('No') );
		#			}
		#		}
		#		$string .= '</tr>';
		#		
		#		$string .= CGI::hidden( -override => 1, -name => $key,
		#				-value => join( ' ', @{$edited{$key}} ) );
		#	}
		#}
		#
		#$string .= end_table() . '</p><p>' .
		#		$c->save_cancel_buttons_l10n_wrapper("$myself?mode=cfgdip" ) . '</p>' .
		#		CGI::end_form();
		#
		#return $string;
	}

	#$html .= CGI::h3(_('Dynamic IP Change Configuration'));
	#$html .= CGI::p($msg);
	#$html .= CGI::p($c->return_link_l10n_wrapper("/netwinder/plugins/os-dyndns/os-dyndns.cgi?mode=cfgdip&emailbutton=$emailbutt"));
	return $html . do_main();
}

sub save (%) 
{
	my $emailbutt = CGI::param('emailbutton');
	# Get all the arguments.
	my %args;
	foreach my $arg ( CGI::param() )
	{
		my @list = CGI::param ($arg);
		if ( $#list > 0 )
		{
			$args {$arg} = \@list;
		}
		else
		{
			$args {$arg} = $list[0];
		}
	}
	
	my %config = ipchange_config();
	
	# Put the arguments into the config hash
	while ( my ($key, $value) = each %args )
	{
		$config{$key} = $value if ( exists $config{$key} );
	}
	
	if ( ipchange_write_config( %config ) )
	{
		return actions_config("emailbutton=$emailbutt");
	}
	else # This should never happen, but it isn't a perfect world
	{
		return CGI::p(CGI::i(_('There was an error writing the configuration file.'))) . actions_config();
	}
}

# Takes a reference to the CGI arguments and returns a 
# hash which is identical to the configuration hash
sub argsToConfig (%) 
{
	my %args = @_;
	my %config = ipchange_config();
	my %done = (); # Which options have we stored
	
	# Mungle the email address
	if ( exists $args{email} )
	{
		if ( $args{'emailbutton'} eq 'external' )
		{
			$config{email} = $args{extemail};
		}
		else
		{
			$config{email} = $args{email};
		}
		
		$done{email} = 1;
	}
	
	# Set the boolean options correctly
	# If an option is unset, it doesn't appear. If it is set, it does appear
	my @valueNames = ipchange_value_names();
	my %boolOpts = ipchange_boolean_options();
	foreach my $key (sort keys %boolOpts )
	{
		my @list = (0, 0);
		
		# If at least some of the options have been supplied, reformat them to the list
		if ( exists $args{$key} )
		{
			my $value = $args{$key};
			
			# If the value isn't a list, make it one
			$value = [ $value ] if ( ! ref( $value ) );
			
			# Do some work to put the list into the proper format
			# arg => [ 'some name', 'another name' ] turns into args => [ 0, 1 ] 
			for (my $count = 0; $count <= $#valueNames; ++ $count)
			{
				$list[$count] = 0;
				# If we find the value name in the CGI's arguments, make it true
				foreach my $arg ( @$value )
				{
					$list[$count] = 1, last if ( $valueNames[$count] eq $arg );
				}
			}
		}
		
		# The list is in the standard format, add it to the configuration
		$config{$key} = \@list;
		$done{$key} = 1;
	}
	
	# Get any remaining options
	foreach my $key (%config)
	{
		$config{$key} = $args{$key} if
			( ! exists $done{$key} and exists $args{$key} );
	}
	
	return %config;
}

# The following subroutings are dynamic dns related 

# dhs2go_config: this shows the Dynamic DNS Configuration page

sub dns2go_config () {

        my $dns2goLink = "javascript:browse('http://www.dns2go.com/Signup')";
        my %dns2goDomains = (   '.dns2go.com' => '.dns2go.com',
                                '.27south.com' => '.27south.com',
                                '.idlegames.com' => '.idlegames.com',
                                '.mygamesite.net' => '.mygamesite.net',
                                '.d2g.com' => '.d2g.com',
                                '.dn2gooffice.com' => '.dns2gooffice.com',
                                '.idleplay.net' => '.idleplay.net',
                                '.101main.net' => '.101main.net',
                                '.101main.com' => '.101main.com',
                                '.nameyourspace.net' => '.nameyourspace.net',
                                '.linux-dude.com' => '.linux-dude.com',
                                '.audio-stream.net' => '.audio-stream.net',
                                '.myftpsite.net' => '.myftpsite.net',
                                '.my-net-space.net' => '.my-net-space.net',
                                '.dynamic-site.net' => '.dynamic-site.net',
                                '.linux-dude.net' => '.linux-dude.net',
                                '.linux-site.net' => '.linux-site.net'   );

	my $address = dns2go_email_addr();
        my $d2gfqdn = dns2go_domain_name();
        my $d2gkey = dns2go_key();

        my $d2ghost = "";
	my $d2gdomain = ""; 
        my $d2gkey1 = "";
        my $d2gkey2 = "";
        my $d2gkey3 = "";
        my $d2gkey4 = "";
        my $d2gemail = "";

	if ($d2gfqdn) {
                $d2gfqdn =~ /([\w\-]+)(.*)/;
                $d2ghost = $1;
                $d2gdomain = $2;
	} else {
		$d2gdomain = '.dns2go.com';
	}

	if ($d2gkey) {
                $d2gkey =~ /(\d{6})-(\w{5})-(\w{5})-(\d{5,6})/;
                $d2gkey1 = $1;
                $d2gkey2 = $2;
                $d2gkey3 = $3;
                $d2gkey4 = $4;
	}

        $d2gemail = dns2go_email_addr();
 
        my $html = "";

        $html .= h3($modes{$mode}[1] );
        $html .= CGI::start_form (-action => CGI::url( -relative=>1 ), -method => 'get');
        $html .= CGI::hidden( -override => 1, -name => 'mode', -value => 'modddns', -align => 'left');

        my $tbl = start_table();

        $tbl .= CGI::Tr(th({-colspan=>2}, _('DynamicDNS Settings')));

        $tbl .= CGI::Tr(CGI::td({-colspan=>2},
                        CGI::p(CGI::b(_('Step 1: '), CGI::a({href=>$dns2goLink}, _('Register with DNS2Go')))),
                        _('To use the DynamicDNS Services of DNS2Go, you must first register with them.'),
                        CGI::a({href=>$dns2goLink}, _('This link')),
                        _('will take you to the DNS2Go registration page. ') .
                        _('Simply follow the steps to register your new DynamicDNS record. (You don\'t need to download the dns2go client.)') . 
                        '<br>&nbsp;'));
        $tbl .= CGI::Tr(CGI::td({-colspan=>2},
                        CGI::p(CGI::b(_('Step 2: Enter your Information'))),
                        CGI::p(_('Use the space below to enter the DNS2Go DynamicDNS information.'))));

        $tbl .= Tr(te({-width=>'38%'}, _('Your DNS2Go domain')).
                te(CGI::textfield(-name=>'d2ghost', -value=>$d2ghost) .
                   CGI::popup_menu(-name=>'d2gdomain',
					-values => [ sort keys %dns2goDomains ],
					-default => $d2gdomain,
					-labels =>\%dns2goDomains)));

        $tbl .= Tr(te({-width=>'38%'}, _('Registration key')) .
                te(CGI::textfield(-name => 'd2gkey1',
					-value => $d2gkey1,
					-size => 6,
					-maxlength => 6)
		   .  '-' .
		   CGI::textfield(-name => 'd2gkey2',
                                        -value => $d2gkey2,
                                        -size => 6,
                                        -maxlength => 5)
                    .  '-' .
		    CGI::textfield(-name => 'd2gkey3',
                                        -value => $d2gkey3,
                                        -size => 6,
                                        -maxlength => 5)
                    .  '-' .
	   	    CGI::textfield(-name => 'd2gkey4',
                                        -value => $d2gkey4,
                                        -size => 6,
                                        -maxlength => 6)));

        $tbl .= Tr(te({-width=>'38%'}, _('Registered e-mail address'). CGI::br().
                                _('(where the DNS2Go e-mail is sent)')),
                   te(CGI::textfield(-name=>'d2gemail',
                                        -value => $d2gemail,
                                        -size=>42,
                                        -maxlength=>60)));
        $tbl .= end_table();
        $html .= $tbl;
        $html .= $c->save_cancel_buttons_l10n_wrapper( CGI::url(-relative=>1) );
        $html .= CGI::end_form();
        $html .= $help_link;
        return $html;
}

# dns2go_modify: this will modify the /etc/dyndns/dns2go.conf. It will write/over-write the
# registered domain name, the registration key, and the registered e-mail address
# to the file

sub dns2go_modify() {
        my $q = new CGI;
        my $host = $q->param('d2ghost');
        my $domain = $q->param('d2gdomain');
        my $key1 = $q->param('d2gkey1');
        my $key2 = $q->param('d2gkey2');
        my $key3 = $q->param('d2gkey3');
        my $key4 = $q->param('d2gkey4');
        my $email = $q->param('d2gemail');

	@errors = ();
	push @errors, _('The domain name is invalid') if ( $host !~ /\w+/ );
	push @errors, _('The registration key is invalid') 
		if ( $key1 !~ /\d{6}/ || $key2 !~ /\w{5}/ || $key3 !~ /\w{5}/ || $key4 !~ /\d{5,6}/ );
	push @errors, _('The e-mail address is invalid') if ( $email !~ /^\w[\w\.\-\_]*@\w[\w\.\-\_]+\w+$/ );

	if (@errors) {
		my $html .= CGI::h3(_('Your request could not be completed because'));
		$html .= CGI::ul(CGI::li(\@errors));
		$html .= CGI::p(CGI::i(_('Please correct the error(s) and try again')));
		$html .= hr(); 
		return  $html . dns2go_config();
	}

        my $fqdn = $host . $domain;
        my $key = $key1 . '-' . uc $key2 . '-' . uc $key3 . '-' . $key4;

	# save to the configuration file.
	my $header = "#\n# Simple configuration file for DNS2Go linux client\n"
                . "#\n# This is a machine generated file. Do not edit.\n#\n";

        open (FH, ">$CONFFILE");
        print FH $header;
        print FH "\n# DNS2Go Server\nserver = discovery.dns2go.com\n";
        print FH "\n# Log file\nlog = $LOGFILE\n";
        print FH "\n# Fully qualified domain name (required)\ndomain = $fqdn\n";
        print FH "\n# Key (required)\nkey = $key\n";
        print FH "\n# Email: $email\n";

        close(FH);
        return do_main();

	# Confirmation for requested changes
	#my @new = ($fqdn, $key, $email);
	#my @old = (dns2go_domain_name(), dns2go_key(), dns2go_email_addr());
	#my @names = (_('DNS2Go domain'), _('Registration key'), _('Registered e-mail address'));

	#my $i;
	#my @changes = ();
	#for ($i=0; $i <= $#names; $i++) {
	#	$old[$i] = 'none' if $old[$i] eq '';
	#	if ($i != $#names) {
	#		push @changes, [ $names[$i], $old[$i], $new[$i] ]
	#			if $old[$i] ne $new[$i];
	#	} else {
	#		$old[$i] =~ /(.*)\@(.*)/;
        #		my $old_ename = $1;
        #		my $old_edomain = $2;

        #		$new[$i] =~ /(.*)\@(.*)/;
        #		my $ename = $1;
        #		my $edomain = $2;
	#		push @changes, [ $names[$i], $old[$i] , $new[$i] ] 
	#			if ($old_ename ne $ename or $old_edomain ne $edomain);
	#	} 
	#}

	#if (not @changes) {
        #        return CGI::h3(_("No changes were requested")) .
        #                $c->return_link_l10n_wrapper('/netwinder/plugins/os-dyndns/os-dyndns.cgi?mode=cfgddns');
	#} else {
	#	my $html = CGI::h3(_('Changed Settings'));
	#	$html .= start_table() .
	#		Tr(th([_('Setting','Old Value','New Value') ])) .
        #        	join ('', map Tr(td(\@$_)), @changes) .
        #        	end_table() ;
	#	$html .= CGI::start_form(-method=>'post',-action=>$myself) .
	#		CGI::hidden(-name=>'mode', -default=>'saveddns', -override=>1) .
	#		CGI::hidden(-name=>'fqdn', -value=>$fqdn) .
	#		CGI::hidden(-name=>'key', -value=>$key) .
	#		CGI::hidden(-name=>'email', -value=>$email) .
	#		$c->save_cancel_buttons_l10n_wrapper(CGI::url(-relative=>1)) .
        #        	CGI::end_form();
	#	return $html;
	#}
}

sub dns2go_save {
	my $fqdn = CGI::param('fqdn');
	my $key = CGI::param('key');
	my $email = CGI::param('email');

	my $header = "#\n# Simple configuration file for DNS2Go linux client\n"
		. "#\n# This is a machine generated file. Do not edit.\n#\n";
		
       	open (FH, ">$CONFFILE");
	print FH $header;
	print FH "\n# DNS2Go Server\nserver = discovery.dns2go.com\n";
	print FH "\n# Log file\nlog = $LOGFILE\n";
        print FH "\n# Fully qualified domain name (required)\ndomain = $fqdn\n";
        print FH "\n# Key (required)\nkey = $key\n";
        print FH "\n# Email: $email\n";

        close(FH);
        return dns2go_config();
}

sub read_up_time() {

	my $upTime;
	my $deltaTime;
	my $secAday = 3600*24;
	my ($days, $hours, $minutes) = (0, 0, 0);

	if ((-e $TIMEFILE)) {
		$upTime = `cat /var/dns2go/uptime`;
		print STDERR "\t upTime: $upTime.\n"
		if $DEBUG;
		$upTime =~ /(.*)/;
		$upTime = $1;
		print STDERR "\t upTime: $upTime.\n"
		if $DEBUG;
		$deltaTime = time - $upTime;
		print STDERR "\t deltaTime: $deltaTime.\n"
		if $DEBUG;
		$days = int ($deltaTime/$secAday);
		$deltaTime -= $days*$secAday;
		$hours = int ($deltaTime/3600);
		$deltaTime -= $hours*3600;
		$minutes = int ($deltaTime/60);
	}
	return ($days, $hours, $minutes);
} # read_up_time

sub write_up_time() {
        my $curTime = time;
        open OUT, ">$TIMEFILE";
        print OUT $curTime;
        close OUT;
}

sub get_runflag() {
	return (-e $RUNFILE) ? 1 : 0;
}

sub get_dns2go_pid() {
	if (-s $STATFILE) {
        	my $d2gpid = `grep ^pid $STATFILE`;
        	$d2gpid =~ s/^.*=\s*//;
        	return ($d2gpid == 0 ? 0 : 1);
	}
	return 0;
}

sub dns2go_email_addr() {
        my $emailAddress;
	if (-s $CONFFILE) {
		$emailAddress = `grep Email $CONFFILE`;
        	$emailAddress =~ s/^\#\s*Email:\s*//;
		chop $emailAddress;
	}
        return $emailAddress;
}

sub dns2go_domain_name() {
	my $fqdn;
	if (-s $CONFFILE) {
		$fqdn = `grep ^domain $CONFFILE`;
		$fqdn =~ s/^domain\s*=\s*//;
		chop $fqdn;
	}
	return $fqdn;
}

sub dns2go_key() {
	my $key;
	if (-s $CONFFILE) {
		$key = `grep ^key $CONFFILE`;
		$key =~ s/^key\s*=\s*//;
		chop $key;
	}
	return $key;
}

sub status_link() {
        my ($lnk, $msg);
        my $href = "os-dyndns.cgi";

        if($_[0] eq "dns2go") {
                if (!(dns2go_domain_name() and dns2go_key())) { # dns2go is not configured yet.
                        $lnk = _('stopped');
                        return $lnk;
                }

                # dns2go is configured properly.
                if (get_dns2go_pid()) {
			system("/bin/touch $RUNFILE > /dev/null");
                        $msg = _('You are about to STOP the $1 service.', $_[1]) . '\n' 
                                . _('Click OK to confirm.');
                        $lnk = CGI::a({-href=>"$myself?mode=actd2g&action=stop",
                        -onClick=>"if(confirm(\'$msg\')!=1){return false;}"}, _('running'));
                } else {
			system("/bin/rm $RUNFILE > /dev/null") if (-e $RUNFILE);
                        $msg = _('You are about to START the $1 service.', $_[1]) . '\n'
                                . _('Click OK to confirm.');
                        $lnk = CGI::a({-href=>"$myself?mode=actd2g&action=start",
                        -onClick=>"if(confirm(\'$msg\')!=1){return false;}"}, _('stopped'));
                }

                return $lnk;
        }

	if($_[0] eq "ipchange") {
		if (-e "/var/lock/ipchange") {
			$msg = _('You are about to STOP the $1 service.', $_[1]) . '\n'
                                . _('Click OK to confirm.');
                        $lnk = CGI::a({-href=>"$myself?mode=actdip&action=stop",
                        -onClick=>"if(confirm(\'$msg\')!=1){return false;}"}, _('running'));
                } else {
                        $msg = _('You are about to START the $1 service.', $_[1]) . '\n'
                                . _('Click OK to confirm.');
                        $lnk = CGI::a({-href=>"$myself?mode=actdip&action=start",
                        -onClick=>"if(confirm(\'$msg\')!=1){return false;}"}, _('stopped'));
                }

                return $lnk;
        } 
}

sub dns2go_action() {
	my $timeout = 5;
	my $html = "";

        my $action = CGI::param('action') || 'stop';
	@errors = ();
        $>=0;
        $<=0;
        if (get_dns2go_pid() and $action eq 'stop') {
		unlink("$RC3_D_DIR/S96dns2go");
                system("/usr/bin/dns2go -k > /dev/null");
                sleep $timeout;
                if (get_dns2go_pid()) {
                        push @errors, _('the system failed in stopping DNS2Go.');
                } else {
                	system("/bin/rm $RUNFILE > /dev/null");
		}
        } elsif (!get_dns2go_pid() and $action eq 'start') {
		# check dns2go service and authentication
                if (system("/usr/bin/dns2go -s > /dev/null")) {

			push @errors, _('the DNS2Go service is probably down.');
		} elsif (!@errors and system("/usr/bin/dns2go -c > /dev/null")) {
			push @errors, _('the domain or key in your configuration is wrong.');
		}

		# no problem, start dns2go
		if (!@errors) {
			system("/usr/bin/dns2go > /dev/null");
			symlink("/etc/rc.d/init.d/dns2go", "$RC3_D_DIR/S96dns2go");
			sleep $timeout;
			if (!get_dns2go_pid()) {
				push @errors, _('the system failed in starting DNS2Go.');
			} else {
				system("/bin/touch $RUNFILE > /dev/null");
			}
		}
	}

	if (@errors) {
		$html .= CGI::h3(_('Your request could not be completed because'));
		$html .= CGI::ul(CGI::li(\@errors));
		$html .= CGI::p(CGI::i(_('Please check your configuration and try again')));
		$html .= hr();
                                            
	}
	return $html . do_main();
}

sub ipchange_action {
	my $action = CGI::param('action') || 'stop';
        @errors = ();
        $>=0;
        $<=0;
	if ($action eq 'stop') {
		system("/bin/rm /var/lock/ipchange > /dev/null");
	} else {
		system("/bin/touch /var/lock/ipchange > /dev/null");
	}
	return do_main();
}
#! /usr/bin/perl -w
package common_plugin;

use strict;
use nameshash;
use CGI;

BEGIN {

  if(!defined(&nameshash::set_LC_MESSAGES_domain)) {
	*common_plugin::__ = *main::__ = \&common_plugin::dummy_gettext_wrapper;
  }
  else {
	*common_plugin::__ = \&main::__;
  }

}

sub new {

  my $proto = shift;
  my $type = ref($proto) || $proto;
  my $self = {};
  $self->{PLUGIN_NAME} = shift;
  bless $self, $type;

}

# Ensures the appropriate l10n vars are defined.

# Note: This function also pushs $core_lang_dir into the main
# namespace.  This is varible is used to specify which relative
# directory documentation and images are in (ie: "" for OS 2.2, and
# "$OS_LANG/" for OS >= 2.4).

sub define_l10n_vars {

  # Ensure $main::LC_DOMAIN is defined:
  if (!defined($main::LC_DOMAIN)) {
    $main::LC_DOMAIN = 'officeserver';
  }
  # Ensure $main::OS_LANG is defined:
  if (!defined($main::OS_LANG)) {
    $main::OS_LANG = 'en_US';
	$main::core_lang_dir = "";
  }
  else {
	$main::core_lang_dir = "$main::OS_LANG/";
  }
  # Ensure $main::OS_LANG_DEF is defined:
  if (!defined($main::OS_LANG_DEF)) {
	$main::OS_LANG_DEF = 'en_US';
  }

}

# Create Plug-in help link:

sub plugin_help_link {

  my $self = shift;
  my $window_name = shift;
  my $start_page = shift;
  my $link_label = shift;

  my $no_doc_msg = __('The required plugin language pack ($1) is not installed.', $main::OS_LANG);

  # If the $main::OS_LANG documentation does not exist, use $main::OS_LANG_DEF.
  # If $main::OS_LANG_DEF documentation does not exist, show an appropriate dialog.

  my $doc_index = "doc/$main::OS_LANG/$start_page";
  if (! -f $doc_index) {
	$doc_index = "doc/$main::OS_LANG_DEF/$start_page";
  }
  my $exists = 1;
  if (! -f $doc_index) {
	$exists = 0;
  }

  my $javascript = <<EOF;

function help_$window_name () {
  if($exists) {
	window.open("$doc_index","$window_name",
                "toolbar=1,statusbar=1,scrollbars=1,resizable=1,width=680,height=500");
  }
  else {
	alert("$no_doc_msg");
  }
}
EOF

  return
	CGI::script($javascript) . &main::hr() .
	  CGI::p(CGI::a({-href=>"javascript:help_$window_name()"}, $link_label));

}

sub l10n_setup {

  my $self = shift;
  my $name = shift;

  # localization setup:
  if(defined(&nameshash::set_LC_MESSAGES_domain)) {
    nameshash::set_LC_MESSAGES_domain($name);
  }

}

sub start_gui_l10n_wrapper {

  my $self = shift;

  $self->l10n_setup($main::LC_DOMAIN);
  my $result = &main::start_gui(@_);
  $self->l10n_setup($self->{PLUGIN_NAME});

  return $result;

}

sub return_link_l10n_wrapper {

  my $self = shift;

  $self->l10n_setup($main::LC_DOMAIN);
  my $result = &main::return_link(@_);
  $self->l10n_setup($self->{PLUGIN_NAME});

  return $result;

}

sub save_cancel_buttons_l10n_wrapper {

  my $self = shift;

  $self->l10n_setup($main::LC_DOMAIN);
  my $result = &main::save_cancel_buttons(@_);
  $self->l10n_setup($self->{PLUGIN_NAME});

  return $result;

}

# -----

# class methods:

sub dummy_gettext_wrapper {

  my @args = @_;
  my @results;

  while(@args > 0) {
    my($args_used, $trans) = dummy_gettext_wrapper_atomic(@args);
    push @results, $trans;
    splice(@args, 0, $args_used);
  }

  if(wantarray) {
    return @results;
  }
  else {
    local $" = ' ';
    return "@results";
  }

}

sub dummy_gettext_wrapper_atomic {

  my $text = shift;

  # outer test for efficiency.
  if($text !~ m/(?:^|[^\$])\$[0-9]/) {
    # normal case.
    # skipping translation.
    return (1, $text);
  }
  else {
    # special "delayed interpolation" case.

    my %args;
    while($text =~ m/(?:\G|[^\$])\$([0-9])/g) {
      if(!exists($args{$1})) {
        $args{$1} = shift;
      }
    }

    # skipping translation.
    my $trans = $text;

    # now, do the interpolation.
    foreach my $i (sort keys %args) {
      $trans =~ s/\$$i/$args{$i}/g;
    }

    my @keys = keys %args;
    return (1 + @keys, $trans);
  }

}

1;
